@@ -16,6 +16,8 @@ import json
 
 from wsme import types as wtypes
 
+from mistral_lib import utils
+
 
 class Resource(wtypes.Base):
     """REST API Resource."""
@@ -27,27 +29,43 @@ class Resource(wtypes.Base):
 
         for attr in self._wsme_attributes:
             attr_val = getattr(self, attr.name)
+
             if not isinstance(attr_val, wtypes.UnsetType):
                 d[attr.name] = attr_val
 
         return d
 
     @classmethod
-    def from_dict(cls, d):
+    def from_tuples(cls, tuple_iterator):
         obj = cls()
 
-        for key, val in d.items():
-            if hasattr(obj, key):
-                setattr(obj, key, val)
+        for col_name, col_val in tuple_iterator:
+            if hasattr(obj, col_name):
+                # Convert all datetime values to strings.
+                setattr(obj, col_name, utils.datetime_to_str(col_val))
 
         return obj
 
+    @classmethod
+    def from_dict(cls, d):
+        return cls.from_tuples(d.items())
+
+    @classmethod
+    def from_db_model(cls, db_model, fields=()):
+        if isinstance(db_model, tuple):
+            db_tuples = zip(fields, db_model)
+        else:
+            db_tuples = db_model.iter_columns(fields=fields)
+
+        return cls.from_tuples(db_tuples)
+
     def __str__(self):
         """WSME based implementation of __str__."""
 
         res = "%s [" % type(self).__name__
 
         first = True
+
         for attr in self._wsme_attributes:
             if not first:
                 res += ', '
@@ -81,18 +99,18 @@ class ResourceList(Resource):
     @classmethod
     def convert_with_links(cls, resources, limit, url=None, fields=None,
                            **kwargs):
-        resource_collection = cls()
+        resource_list = cls()
 
-        setattr(resource_collection, resource_collection._type, resources)
+        setattr(resource_list, resource_list._type, resources)
 
-        resource_collection.next = resource_collection.get_next(
+        resource_list.next = resource_list.get_next(
             limit,
             url=url,
             fields=fields,
             **kwargs
         )
 
-        return resource_collection
+        return resource_list
 
     def has_next(self, limit):
         """Return whether resources has more items."""
@@ -103,9 +121,14 @@ class ResourceList(Resource):
         if not self.has_next(limit):
             return wtypes.Unset
 
-        q_args = ''.join(
-            ['%s=%s&' % (key, value) for key, value in kwargs.items()]
-        )
+        q_args = ''
+
+        for key, value in kwargs.items():
+            if isinstance(value, dict):
+                q_args += '%s=%s:%s&' % \
+                          (key, list(value.keys())[0], list(value.values())[0])
+            else:
+                q_args += '%s=%s&' % (key, value)
 
         resource_args = (
             '?%(args)slimit=%(limit)d&marker=%(marker)s' %
@@ -149,8 +172,9 @@ class Link(Resource):
 
     href = wtypes.text
     target = wtypes.text
+    rel = wtypes.text
 
     @classmethod
     def sample(cls):
         return cls(href='http://example.com/here',
-                   target='here')
+                   target='here', rel='self')
