@@ -39,6 +39,7 @@ def auth_enable_check(func):
             msg = ("Resource sharing feature can only be supported with "
                    "authentication enabled.")
             raise exc.WorkflowException(msg)
+
         return func(*args, **kwargs)
 
     return wrapped
@@ -58,7 +59,7 @@ class MembersController(rest.RestController):
         """Shows resource member details."""
         acl.enforce('members:get', context.ctx())
 
-        LOG.info(
+        LOG.debug(
             "Fetch resource member [resource_id=%s, resource_type=%s, "
             "member_id=%s].",
             self.resource_id,
@@ -66,13 +67,17 @@ class MembersController(rest.RestController):
             member_id
         )
 
-        member_dict = db_api.get_resource_member(
+        # Use retries to prevent possible failures.
+        r = rest_utils.create_db_retry_object()
+
+        member_db = r.call(
+            db_api.get_resource_member,
             self.resource_id,
             self.type,
             member_id
-        ).to_dict()
+        )
 
-        return resources.Member.from_dict(member_dict)
+        return resources.Member.from_db_model(member_db)
 
     @rest_utils.wrap_pecan_controller_exception
     @auth_enable_check
@@ -81,7 +86,7 @@ class MembersController(rest.RestController):
         """Return all members with whom the resource has been shared."""
         acl.enforce('members:list', context.ctx())
 
-        LOG.info(
+        LOG.debug(
             "Fetch resource members [resource_id=%s, resource_type=%s].",
             self.resource_id,
             self.type
@@ -91,9 +96,10 @@ class MembersController(rest.RestController):
             self.resource_id,
             self.type
         )
+
         members = [
-            resources.Member.from_dict(member.to_dict())
-            for member in db_members
+            resources.Member.from_db_model(db_member)
+            for db_member in db_members
         ]
 
         return resources.Members(members=members)
@@ -109,7 +115,7 @@ class MembersController(rest.RestController):
         """Shares the resource to a new member."""
         acl.enforce('members:create', context.ctx())
 
-        LOG.info(
+        LOG.debug(
             "Share resource to a member. [resource_id=%s, "
             "resource_type=%s, member_info=%s].",
             self.resource_id,
@@ -118,25 +124,30 @@ class MembersController(rest.RestController):
         )
 
         if not member_info.member_id:
-            msg = "Member id must be provided."
-            raise exc.WorkflowException(msg)
+            raise exc.WorkflowException("Member id must be provided.")
 
-        wf_db = db_api.get_workflow_definition(self.resource_id)
+        @rest_utils.rest_retry_on_db_error
+        def _create_resource_member():
+            with db_api.transaction():
+                wf_db = db_api.get_workflow_definition(self.resource_id)
 
-        if wf_db.scope != 'private':
-            msg = "Only private resource could be shared."
-            raise exc.WorkflowException(msg)
+                if wf_db.scope != 'private':
+                    raise exc.WorkflowException(
+                        "Only private resource could be shared."
+                    )
 
-        resource_member = {
-            'resource_id': self.resource_id,
-            'resource_type': self.type,
-            'member_id': member_info.member_id,
-            'status': 'pending'
-        }
+                resource_member = {
+                    'resource_id': self.resource_id,
+                    'resource_type': self.type,
+                    'member_id': member_info.member_id,
+                    'status': 'pending'
+                }
 
-        db_member = db_api.create_resource_member(resource_member)
+                return db_api.create_resource_member(resource_member)
 
-        return resources.Member.from_dict(db_member.to_dict())
+        db_member = _create_resource_member()
+
+        return resources.Member.from_db_model(db_member)
 
     @rest_utils.wrap_pecan_controller_exception
     @auth_enable_check
@@ -145,7 +156,7 @@ class MembersController(rest.RestController):
         """Sets the status for a resource member."""
         acl.enforce('members:update', context.ctx())
 
-        LOG.info(
+        LOG.debug(
             "Update resource member status. [resource_id=%s, "
             "member_id=%s, member_info=%s].",
             self.resource_id,
@@ -157,14 +168,16 @@ class MembersController(rest.RestController):
             msg = "Status must be provided."
             raise exc.WorkflowException(msg)
 
-        db_member = db_api.update_resource_member(
+        db_member = rest_utils.rest_retry_on_db_error(
+            db_api.update_resource_member
+        )(
             self.resource_id,
             self.type,
             member_id,
             {'status': member_info.status}
         )
 
-        return resources.Member.from_dict(db_member.to_dict())
+        return resources.Member.from_db_model(db_member)
 
     @rest_utils.wrap_pecan_controller_exception
     @auth_enable_check
@@ -173,7 +186,7 @@ class MembersController(rest.RestController):
         """Deletes a member from the member list of a resource."""
         acl.enforce('members:delete', context.ctx())
 
-        LOG.info(
+        LOG.debug(
             "Delete resource member. [resource_id=%s, "
             "resource_type=%s, member_id=%s].",
             self.resource_id,
@@ -181,7 +194,7 @@ class MembersController(rest.RestController):
             member_id
         )
 
-        db_api.delete_resource_member(
+        rest_utils.rest_retry_on_db_error(db_api.delete_resource_member)(
             self.resource_id,
             self.type,
             member_id
