@@ -37,6 +37,7 @@ from mistral.db import utils as m_dbutils
 from mistral.db.v2.sqlalchemy import filters as db_filters
 from mistral.db.v2.sqlalchemy import models
 from mistral import exceptions as exc
+from mistral.services import maintenance as m
 from mistral.services import security
 from mistral.workflow import states
 from mistral_lib import utils
@@ -66,7 +67,10 @@ def setup_db():
 
         try:
             models.Workbook.metadata.create_all(b.get_engine())
-
+            # NOTE(wortellen): We need this in case if we use sqlite3.
+            # We have to fill this table, but I haven't found any other
+            # appropriate place where this can be done.
+            fill_metrics_table()
             _initialized = True
         except sa.exc.OperationalError as e:
             raise exc.DBError("Failed to setup database: %s" % e)
@@ -107,16 +111,17 @@ def end_tx():
 
 @contextlib.contextmanager
 def transaction(read_only=False):
-    start_tx()
+    with b.tx_lock:
+        start_tx()
 
-    try:
-        yield
-        if read_only:
-            rollback_tx()
-        else:
-            commit_tx()
-    finally:
-        end_tx()
+        try:
+            yield
+            if read_only:
+                rollback_tx()
+            else:
+                commit_tx()
+        finally:
+            end_tx()
 
 
 @b.session_aware()
@@ -285,6 +290,13 @@ def _get_collection(model, insecure=False, limit=None, marker=None,
     return query.all()
 
 
+def get_db_objects(model, insecure=False, **filters):
+    query = b.model_query(model) if insecure else _secure_query(model)
+    query = db_filters.apply_filters(query, model, **filters)
+
+    return query
+
+
 def _get_count(model, insecure=False, **filters):
     query = b.model_query(model) if insecure else _secure_query(model)
 
@@ -300,6 +312,11 @@ def _get_db_object_by_name(model, name, columns=()):
 
 
 def _get_db_object_by_id(model, id, insecure=False, columns=()):
+    columns = (
+        tuple([getattr(model, f) for f in columns if hasattr(model, f)])
+        if columns and isinstance(columns, list) else columns
+    )
+
     query = (
         b.model_query(model, columns=columns)
         if insecure
@@ -312,6 +329,10 @@ def _get_db_object_by_id(model, id, insecure=False, columns=()):
 def _get_db_object_by_name_and_namespace_or_id(model, identifier,
                                                namespace=None, insecure=False,
                                                columns=()):
+    columns = (
+        tuple([getattr(model, f) for f in columns if hasattr(model, f)])
+        if columns and isinstance(columns, list) else columns
+    )
     query = (
         b.model_query(model, columns=columns)
         if insecure
@@ -633,6 +654,171 @@ def delete_workflow_definitions(session=None, **kwargs):
     return _delete_all(models.WorkflowDefinition, **kwargs)
 
 
+# Code sources.
+
+@b.session_aware()
+def create_code_source(values, session=None):
+    code_src = models.CodeSource()
+    code_src.update(values.copy())
+
+    try:
+        code_src.save(session=session)
+    except db_exc.DBDuplicateEntry:
+        raise exc.DBDuplicateEntryError(
+            "Duplicate entry for CodeSource ['name', 'namespace',"
+            " 'project_id']: {}, {}, {}".format(code_src.name,
+                                                code_src.namespace,
+                                                code_src.project_id)
+        )
+
+    return code_src
+
+
+@b.session_aware()
+def get_code_sources(fields=None, session=None, **kwargs):
+    return _get_collection(
+        model=models.CodeSource,
+        fields=fields,
+        **kwargs
+    )
+
+
+@b.session_aware()
+def get_code_source(identifier, fields=(), session=None, namespace=''):
+    code_src = _get_db_object_by_name_and_namespace_or_id(
+        models.CodeSource,
+        identifier,
+        namespace=namespace,
+        columns=fields
+    )
+
+    if not code_src:
+        raise exc.DBEntityNotFoundError(
+            "Code Source not found [name=%s,namespace=%s]"
+            % (identifier, namespace)
+        )
+
+    return code_src
+
+
+@b.session_aware()
+def load_code_source(identifier, fields=(), session=None, namespace=''):
+    return _get_db_object_by_name_and_namespace_or_id(
+        models.CodeSource,
+        identifier,
+        namespace=namespace,
+        columns=fields
+    )
+
+
+@b.session_aware()
+def update_code_source(identifier, values, namespace='', session=None):
+    code_src = get_code_source(identifier, namespace=namespace)
+
+    values['version'] = code_src.version + 1
+
+    code_src.update(values.copy())
+
+    return code_src
+
+
+@b.session_aware()
+def delete_code_source(identifier, namespace='', session=None):
+    code_src = get_code_source(
+        identifier,
+        namespace=namespace,
+        session=session
+    )
+
+    session.delete(code_src)
+
+
+@b.session_aware()
+def delete_code_sources(session=None, **kwargs):
+    return _delete_all(models.CodeSource, **kwargs)
+
+
+# Dynamic actions.
+
+@b.session_aware()
+def create_dynamic_action_definition(values, session=None):
+    action_def = models.DynamicActionDefinition()
+
+    action_def.update(values.copy())
+
+    try:
+        action_def.save(session=session)
+    except db_exc.DBDuplicateEntry:
+        raise exc.DBDuplicateEntryError(
+            "Duplicate entry for dynamic action definition "
+            "[name=%s, namespace=%s, project_id=%s]"
+            % (action_def.name, action_def.namespace, action_def.project_id)
+        )
+
+    return action_def
+
+
+@b.session_aware()
+def update_dynamic_action_definition(identifier, values, namespace='',
+                                     session=None):
+    action_def = get_dynamic_action_definition(identifier, namespace=namespace)
+
+    action_def.update(values.copy())
+
+    return action_def
+
+
+@b.session_aware()
+def get_dynamic_action_definition(identifier, fields=(), namespace='',
+                                  session=None):
+    action = _get_db_object_by_name_and_namespace_or_id(
+        models.DynamicActionDefinition,
+        identifier,
+        namespace=namespace,
+        columns=fields
+    )
+
+    if not action:
+        raise exc.DBEntityNotFoundError(
+            "Dynamic Action not found [name=%s,namespace=%s]"
+            % (identifier, namespace)
+        )
+
+    return action
+
+
+@b.session_aware()
+def load_dynamic_action_definition(identifier, fields=(), namespace='',
+                                   session=None):
+    return _get_db_object_by_name_and_namespace_or_id(
+        models.DynamicActionDefinition,
+        identifier,
+        namespace=namespace,
+        columns=fields
+    )
+
+
+@b.session_aware()
+def get_dynamic_action_definitions(fields=None, session=None, **kwargs):
+    return _get_collection(
+        model=models.DynamicActionDefinition,
+        fields=fields,
+        **kwargs
+    )
+
+
+@b.session_aware()
+def delete_dynamic_action_definition(identifier, namespace='', session=None):
+    action_def = get_dynamic_action_definition(identifier, namespace)
+
+    session.delete(action_def)
+
+
+@b.session_aware()
+def delete_dynamic_action_definitions(session=None, **kwargs):
+    return _delete_all(models.DynamicActionDefinition, **kwargs)
+
+
 # Action definitions.
 
 @b.session_aware()
@@ -659,6 +845,7 @@ def get_action_definition(identifier, fields=(), session=None, namespace=''):
         namespace=namespace,
         columns=fields
     )
+
     #  If the action was not found in the given namespace,
     #  look in the default namespace
     if not a_def:
@@ -671,7 +858,7 @@ def get_action_definition(identifier, fields=(), session=None, namespace=''):
 
     if not a_def:
         raise exc.DBEntityNotFoundError(
-            "Action definition not found [action_name=%s,namespace=%s]"
+            "Action definition not found [action_name=%s, namespace=%s]"
             % (identifier, namespace)
         )
 
@@ -748,6 +935,10 @@ def delete_action_definitions(session=None, **kwargs):
 
 @b.session_aware()
 def get_action_execution(id, insecure=False, fields=(), session=None):
+    # Allow admin to retrieve action execution by overwriting insecure
+    ctx = context.ctx()
+    insecure = ctx.is_admin or insecure
+
     a_ex = _get_db_object_by_id(models.ActionExecution, id, insecure=insecure,
                                 columns=fields)
 
@@ -977,7 +1168,11 @@ def update_workflow_execution_state(id, cur_state, state):
 
 @b.session_aware()
 def get_task_execution(id, fields=(), session=None):
-    task_ex = _get_db_object_by_id(models.TaskExecution, id, columns=fields)
+    # Allow admin to retrieve all tasks
+    ctx = context.ctx()
+
+    task_ex = _get_db_object_by_id(models.TaskExecution, id,
+                                   insecure=ctx.is_admin, columns=fields)
 
     if not task_ex:
         raise exc.DBEntityNotFoundError(
@@ -1015,6 +1210,7 @@ def _get_completed_task_executions_query(kwargs):
         models.TaskExecution.state.in_(
             [states.ERROR,
              states.CANCELLED,
+             states.SKIPPED,
              states.SUCCESS]
         )
     )
@@ -1426,13 +1622,14 @@ def _get_completed_root_executions_query(columns):
 
 
 @b.session_aware()
-def get_cron_trigger(identifier, session=None):
+def get_cron_trigger(identifier, session=None, fields=()):
     ctx = context.ctx()
 
     cron_trigger = _get_db_object_by_name_and_namespace_or_id(
         models.CronTrigger,
         identifier,
-        insecure=ctx.is_admin
+        insecure=ctx.is_admin,
+        columns=fields,
     )
 
     if not cron_trigger:
@@ -1444,10 +1641,11 @@ def get_cron_trigger(identifier, session=None):
 
 
 @b.session_aware()
-def get_cron_trigger_by_id(id, session=None):
+def get_cron_trigger_by_id(id, session=None, fields=()):
     ctx = context.ctx()
     cron_trigger = _get_db_object_by_id(models.CronTrigger, id,
-                                        insecure=ctx.is_admin)
+                                        insecure=ctx.is_admin,
+                                        columns=fields)
     if not cron_trigger:
         raise exc.DBEntityNotFoundError(
             "Cron trigger not found [id=%s]" % id
@@ -1457,10 +1655,11 @@ def get_cron_trigger_by_id(id, session=None):
 
 
 @b.session_aware()
-def load_cron_trigger(identifier, session=None):
+def load_cron_trigger(identifier, session=None, fields=()):
     return _get_db_object_by_name_and_namespace_or_id(
         models.CronTrigger,
-        identifier
+        identifier,
+        columns=fields,
     )
 
 
@@ -1571,8 +1770,8 @@ def delete_cron_triggers(session=None, **kwargs):
 # Environments.
 
 @b.session_aware()
-def get_environment(name, session=None):
-    env = _get_db_object_by_name(models.Environment, name)
+def get_environment(name, session=None, fields=()):
+    env = _get_db_object_by_name(models.Environment, name, columns=fields)
 
     if not env:
         raise exc.DBEntityNotFoundError(
@@ -1583,8 +1782,8 @@ def get_environment(name, session=None):
 
 
 @b.session_aware()
-def load_environment(name, session=None):
-    return _get_db_object_by_name(models.Environment, name)
+def load_environment(name, session=None, fields=()):
+    return _get_db_object_by_name(models.Environment, name, columns=fields)
 
 
 @b.session_aware()
@@ -1809,8 +2008,9 @@ def _get_accepted_resources(res_type):
 # Event triggers.
 
 @b.session_aware()
-def get_event_trigger(id, insecure=False, session=None):
-    event_trigger = _get_db_object_by_id(models.EventTrigger, id, insecure)
+def get_event_trigger(id, insecure=False, session=None, fields=()):
+    event_trigger = _get_db_object_by_id(models.EventTrigger, id, insecure,
+                                         columns=fields)
 
     if not event_trigger:
         raise exc.DBEntityNotFoundError(
@@ -1821,8 +2021,9 @@ def get_event_trigger(id, insecure=False, session=None):
 
 
 @b.session_aware()
-def load_event_trigger(id, insecure=False, session=None):
-    return _get_db_object_by_id(models.EventTrigger, id, insecure)
+def load_event_trigger(id, insecure=False, session=None, fields=()):
+    return _get_db_object_by_id(models.EventTrigger, id, insecure,
+                                columns=fields)
 
 
 @b.session_aware()
@@ -1888,6 +2089,13 @@ def delete_event_triggers(session=None, **kwargs):
 
 @b.session_aware()
 def create_named_lock(name, session=None):
+    if b.get_driver_name() == 'sqlite':
+        # In case of 'sqlite' we need to apply a manual lock because sqlite
+        # is not READ_COMMITED able
+        # This is useful for testing purpose as sqlite is not supposed to be
+        # used for prod
+        sqlite_lock.acquire_lock(name, session)
+
     # This method has to work not through SQLAlchemy session because
     # session may not immediately issue an SQL query to a database
     # and instead just schedule it whereas we need to make sure to
@@ -1911,7 +2119,7 @@ def get_named_locks(session=None, **kwargs):
 
 
 @b.session_aware()
-def delete_named_lock(lock_id, session=None):
+def delete_named_lock(lock_id, name, session=None):
     # This method has to work without SQLAlchemy session because
     # session may not immediately issue an SQL query to a database
     # and instead just schedule it whereas we need to make sure to
@@ -1926,6 +2134,9 @@ def delete_named_lock(lock_id, session=None):
 
     session.flush()
 
+    if b.get_driver_name() == 'sqlite':
+        sqlite_lock.release_lock(name, session)
+
 
 @contextlib.contextmanager
 def named_lock(name):
@@ -1939,4 +2150,46 @@ def named_lock(name):
 
     yield
 
-    delete_named_lock(lock_id)
+    delete_named_lock(lock_id, name)
+
+
+@b.session_aware()
+def get_maintenance_status(session=None):
+    maintenance_entry = (
+        session.query(models.MistralMetrics)
+        .filter_by(name="maintenance_status")
+        .first()
+    )
+    return maintenance_entry.value if maintenance_entry else None
+
+
+@b.session_aware()
+def update_maintenance_status(status, session=None):
+    maintenance_entry = session.query(models.MistralMetrics).get(1)
+    maintenance_entry.value = status
+    session.commit()
+
+
+@b.session_aware()
+def fill_metrics_table(session=None):
+    maintenance_entry = session.query(models.MistralMetrics).get(1)
+    if not maintenance_entry:
+        maintenance = models.MistralMetrics()
+        maintenance.update(
+            {
+                'id': 1,
+                'name': 'maintenance_status',
+                'value': m.RUNNING
+            }
+        )
+        maintenance.save(session=session)
+
+
+@b.session_aware()
+def get_overdue_calls(time, session=None):
+    query = b.model_query(models.DelayedCall)
+
+    query = query.filter(models.DelayedCall.execution_time < time)
+    query = query.filter_by(processing=True)
+
+    return query.all()
