@@ -156,6 +156,7 @@ class WorkflowDefinition(Definition):
     """Contains info about workflow (including definition in Mistral DSL)."""
 
     __tablename__ = 'workflow_definitions_v2'
+    checksum = sa.Column(sa.String(32), nullable=True)
     __table_args__ = (
         sa.UniqueConstraint(
             'name',
@@ -219,6 +220,18 @@ class CodeSource(mb.MistralSecureModelBase):
     tags = sa.Column(st.JsonListType())
 
 
+class MistralMetrics(mb.MistralModelBase):
+    """Table for maintenance mode."""
+
+    __tablename__ = 'mistral_metrics'
+    __table_args__ = (
+        sa.UniqueConstraint('name'),
+    )
+    id = sa.Column(sa.Integer, primary_key=True)
+    name = sa.Column(sa.String(255), nullable=False)
+    value = sa.Column(sa.String(255), nullable=False)
+
+
 class DynamicActionDefinition(mb.MistralSecureModelBase):
     """Contains info about registered Dynamic Actions."""
 
