@@ -28,6 +28,7 @@ from mistral.db.v2.sqlalchemy import models as db_models
 from mistral.engine import action_handler
 from mistral.engine import base
 from mistral.engine import post_tx_queue
+from mistral.engine import task_handler
 from mistral.engine import workflow_handler as wf_handler
 from mistral import exceptions
 from mistral.workflow import states
@@ -81,6 +82,13 @@ class DefaultEngine(base.Engine):
 
                 return wf_ex.get_clone()
 
+    @post_tx_queue.run
+    def start_task(self, task_ex_id, first_run, waiting,
+                   triggered_by, rerun, reset, **params):
+        with db_api.transaction():
+            task_handler.run_task(task_ex_id, waiting,
+                                  triggered_by, rerun, reset, first_run)
+
     @db_utils.retry_on_db_error
     @post_tx_queue.run
     def start_action(self, action_name, action_input,
@@ -197,13 +205,19 @@ class DefaultEngine(base.Engine):
 
     @db_utils.retry_on_db_error
     @post_tx_queue.run
-    def rerun_workflow(self, task_ex_id, reset=True, env=None):
+    def rerun_workflow(self, task_ex_id, reset=True, skip=False, env=None):
         with db_api.transaction():
             task_ex = db_api.get_task_execution(task_ex_id)
 
             wf_ex = task_ex.workflow_execution
 
-            wf_handler.rerun_workflow(wf_ex, task_ex, reset=reset, env=env)
+            wf_handler.rerun_workflow(
+                wf_ex,
+                task_ex,
+                reset=reset,
+                skip=skip,
+                env=env
+            )
 
             return wf_ex.get_clone()
 
@@ -240,8 +254,8 @@ class DefaultEngine(base.Engine):
                     db_api.update_action_execution_heartbeat(exec_id)
                 except exceptions.DBEntityNotFoundError:
                     LOG.debug(
-                        "Action execution heartbeat update failed. {}"
-                        .format(exec_id),
+                        "Action execution heartbeat update failed. {}",
+                        exec_id,
                         exc_info=True
                     )
                     # Ignore this error and continue with the
