@@ -16,7 +16,7 @@
 #    limitations under the License.
 
 import abc
-import collections
+from collections import abc as collections_abc
 import copy
 import json
 from oslo_config import cfg
@@ -289,7 +289,7 @@ class Task(object, metaclass=abc.ABCMeta):
         self.reset_flag = True
 
     @profiler.trace('task-set-state')
-    def set_state(self, state, state_info, processed=None):
+    def set_state(self, state, state_info, processed=None, first_run=False):
         """Sets task state without executing post completion logic.
 
         :param state: New task state.
@@ -323,8 +323,8 @@ class Task(object, metaclass=abc.ABCMeta):
             # was WAITING (all preconditions are satisfied and it's
             # ready to start) or IDLE, or the task is being rerun. So
             # we treat all iterations of "retry" policy as one run.
-            if state == states.RUNNING and \
-                    (cur_state in (None, states.WAITING) or self.rerun):
+            if state == states.RUNNING and (cur_state in (
+                    None, states.WAITING, states.IDLE) or self.rerun):
                 self.task_ex.started_at = utils.utc_now_sec()
 
             if states.is_completed(state):
@@ -336,7 +336,10 @@ class Task(object, metaclass=abc.ABCMeta):
             if processed is not None:
                 self.task_ex.processed = processed
 
-            self._notify(cur_state, state)
+            if first_run:
+                self._notify(None, state)
+            else:
+                self._notify(cur_state, state)
 
             wf_trace.info(
                 self.task_ex.workflow_execution,
@@ -351,7 +354,7 @@ class Task(object, metaclass=abc.ABCMeta):
         return True
 
     @profiler.trace('task-complete')
-    def complete(self, state, state_info=None):
+    def complete(self, state, state_info=None, skip=False):
         """Complete task and set specified state.
 
         Method sets specified task state and runs all necessary post
@@ -365,7 +368,7 @@ class Task(object, metaclass=abc.ABCMeta):
         assert self.task_ex
 
         # Ignore if task already completed.
-        if self.is_completed():
+        if self.is_completed() and not states.is_skipped(state):
             return
 
         # If we were unable to change the task state it means that it was
@@ -375,6 +378,8 @@ class Task(object, metaclass=abc.ABCMeta):
         if not self.set_state(state, state_info):
             return
 
+        self._update_inbound_context()
+
         data_flow.publish_variables(self.task_ex, self.task_spec)
 
         if not self.task_spec.get_keep_result():
@@ -383,7 +388,8 @@ class Task(object, metaclass=abc.ABCMeta):
                 if hasattr(ex, 'output'):
                     ex.output = {}
 
-        self._after_task_complete()
+        if not states.is_skipped(state):
+            self._after_task_complete()
 
         # Ignore DELAYED state.
         if self.task_ex.state == states.RUNNING_DELAYED:
@@ -483,9 +489,11 @@ class Task(object, metaclass=abc.ABCMeta):
 
     @profiler.trace('task-create-task-execution')
     def _create_task_execution(self, state=None, state_info=None):
+        task_id = utils.generate_unicode_uuid()
+        task_name = self.task_spec.get_name()
         values = {
-            'id': utils.generate_unicode_uuid(),
-            'name': self.task_spec.get_name(),
+            'id': task_id,
+            'name': task_name,
             'workflow_execution_id': self.wf_ex.id,
             'workflow_name': self.wf_ex.workflow_name,
             'workflow_namespace': self.wf_ex.workflow_namespace,
@@ -505,10 +513,36 @@ class Task(object, metaclass=abc.ABCMeta):
         if self.triggered_by:
             values['runtime_context']['triggered_by'] = self.triggered_by
 
+        LOG.info("Create task execution [task_name=%s, task_ex_id=%s, "
+                 "wf_ex_id=%s]", task_name, task_id, self.wf_ex.id)
+
         self.task_ex = db_api.create_task_execution(values)
 
         self.created = True
 
+    def _update_inbound_context(self):
+        assert self.task_ex
+
+        wf_ctrl = wf_base.get_controller(self.wf_ex, self.wf_spec)
+
+        triggered_by = self._get_triggered_by_ids()
+
+        self.ctx = wf_ctrl.get_task_inbound_context(self.task_spec,
+                                                    triggered_by=triggered_by)
+
+        utils.update_dict(self.task_ex.in_context, self.ctx)
+
+    def _get_triggered_by_ids(self):
+        ids = []
+
+        if "triggered_by" not in self.task_ex.runtime_context:
+            return ids
+
+        for trigger in self.task_ex.runtime_context["triggered_by"]:
+            ids.append(trigger["task_id"])
+
+        return ids
+
     def _get_safe_rerun(self):
         safe_rerun = self.task_spec.get_safe_rerun()
 
@@ -563,41 +597,47 @@ class RegularTask(Task):
         self.update(action_ex.state)
 
     @profiler.trace('task-run')
-    def run(self):
-        if not self.task_ex:
+    def run(self, first_run=False):
+        if first_run:
             self._run_new()
         else:
             self._run_existing()
 
     @profiler.trace('task-run-new')
     def _run_new(self):
+        if self.waiting:
+            return
+
+        if states.is_idle(self.task_ex.state):
+            # Set the RUNNING state and trigger all operations
+            # related to the state change.
+            self.set_state(
+                states.RUNNING, self.task_ex.state_info, first_run=True)
+
+            LOG.debug(
+                'Starting task [name=%s, init_state=%s, workflow_name=%s,'
+                ' execution_id=%s]',
+                self.task_spec.get_name(),
+                self.task_ex.state,
+                self.wf_ex.name,
+                self.wf_ex.id
+            )
+
+            self._before_task_start()
+
+            # Policies could possibly change task state.
+            if self.task_ex.state != states.RUNNING:
+                return
+
+            self._schedule_actions()
+
+    @profiler.trace('task-create-new')
+    def create_new(self):
         if self.waiting:
             self.defer()
-
             return
 
-        self._create_task_execution()
-
-        # Set the initial state and trigger all operations
-        # related to the state change.
-        self.set_state(states.RUNNING, None)
-
-        LOG.debug(
-            'Starting task [name=%s, init_state=%s, workflow_name=%s,'
-            ' execution_id=%s]',
-            self.task_spec.get_name(),
-            self.task_ex.state,
-            self.wf_ex.name,
-            self.wf_ex.id
-        )
-
-        self._before_task_start()
-
-        # Policies could possibly change task state.
-        if self.task_ex.state != states.RUNNING:
-            return
-
-        self._schedule_actions()
+        self._create_task_execution(state=states.IDLE)
 
     @profiler.trace('task-run-existing')
     def _run_existing(self):
@@ -625,15 +665,6 @@ class RegularTask(Task):
         self._reset_actions()
         self._schedule_actions()
 
-    def _update_inbound_context(self):
-        assert self.task_ex
-
-        wf_ctrl = wf_base.get_controller(self.wf_ex, self.wf_spec)
-
-        self.ctx = wf_ctrl.get_task_inbound_context(self.task_spec)
-
-        utils.update_dict(self.task_ex.in_context, self.ctx)
-
     def _update_triggered_by(self):
         assert self.task_ex
 
@@ -667,12 +698,16 @@ class RegularTask(Task):
 
         action = self._build_action()
 
-        action.schedule(
-            input_dict,
-            target,
-            safe_rerun=self._get_safe_rerun(),
-            timeout=self._get_timeout()
-        )
+        try:
+            action.schedule(
+                input_dict,
+                target,
+                safe_rerun=self._get_safe_rerun(),
+                timeout=self._get_timeout()
+            )
+        except exc.MistralException as e:
+            self.complete(states.ERROR, e.message)
+            return
 
     @profiler.trace('regular-task-get-target', hide_args=True)
     def _get_target(self, input_dict):
@@ -863,20 +898,24 @@ class WithItemsTask(RegularTask):
 
             return
 
-        for i, input_dict in input_dicts:
-            target = self._get_target(input_dict)
+        try:
+            for i, input_dict in input_dicts:
+                target = self._get_target(input_dict)
 
-            action = self._build_action()
+                action = self._build_action()
 
-            action.schedule(
-                input_dict,
-                target,
-                index=i,
-                safe_rerun=self._get_safe_rerun(),
-                timeout=self._get_timeout()
-            )
+                action.schedule(
+                    input_dict,
+                    target,
+                    index=i,
+                    safe_rerun=self._get_safe_rerun(),
+                    timeout=self._get_timeout()
+                )
 
-            self._decrease_capacity(1)
+                self._decrease_capacity(1)
+        except exc.MistralException as e:
+            self.complete(states.ERROR, e.message)
+            return
 
     def _get_with_items_values(self):
         """Returns all values evaluated from 'with-items' expression.
@@ -909,7 +948,7 @@ class WithItemsTask(RegularTask):
         required_len = -1
 
         for var, items in exp_res.items():
-            if not isinstance(items, collections.Iterable):
+            if not isinstance(items, collections_abc.Iterable):
                 raise exc.InputException(
                     "Wrong input format for: %s. Iterable type is"
                     " expected for each value." % result
