@@ -68,29 +68,42 @@ class ReverseWorkflowEngineTest(base.EngineTestCase):
         wb_service.create_workbook_v2(WORKBOOK)
 
     def test_start_task1(self):
-        wf_input = {'param1': 'a', 'param2': 'b'}
+        wf_input = {
+            'param1': 'a',
+            'param2': 'b'
+        }
 
         wf_ex = self.engine.start_workflow(
             'my_wb.wf1',
-            wf_input,
+            wf_input=wf_input,
             task_name='task1'
         )
 
         # Execution 1.
         self.assertIsNotNone(wf_ex)
         self.assertDictEqual(wf_input, wf_ex.input)
-        self.assertDictEqual({'task_name': 'task1'}, wf_ex.params)
+        self.assertDictEqual(
+            {
+                'task_name': 'task1',
+                'namespace': '',
+                'env': {}
+            },
+            wf_ex.params
+        )
 
         # Wait till workflow 'wf1' is completed.
         self.await_workflow_success(wf_ex.id)
 
-        wf_ex = db_api.get_workflow_execution(wf_ex.id)
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
-        self.assertEqual(1, len(wf_ex.task_executions))
+            task_execs = wf_ex.task_executions
+
+        self.assertEqual(1, len(task_execs))
         self.assertEqual(1, len(db_api.get_task_executions()))
 
         task_ex = self._assert_single_item(
-            wf_ex.task_executions,
+            task_execs,
             name='task1',
             state=states.SUCCESS
         )
@@ -98,29 +111,42 @@ class ReverseWorkflowEngineTest(base.EngineTestCase):
         self.assertDictEqual({'result1': 'a'}, task_ex.published)
 
     def test_start_task2(self):
-        wf_input = {'param1': 'a', 'param2': 'b'}
+        wf_input = {
+            'param1': 'a',
+            'param2': 'b'
+        }
 
         wf_ex = self.engine.start_workflow(
             'my_wb.wf1',
-            wf_input,
+            wf_input=wf_input,
             task_name='task2'
         )
 
         # Execution 1.
         self.assertIsNotNone(wf_ex)
         self.assertDictEqual(wf_input, wf_ex.input)
-        self.assertDictEqual({'task_name': 'task2'}, wf_ex.params)
+        self.assertDictEqual(
+            {
+                'task_name': 'task2',
+                'namespace': '',
+                'env': {}
+            },
+            wf_ex.params
+        )
 
         # Wait till workflow 'wf1' is completed.
         self.await_workflow_success(wf_ex.id)
 
-        wf_ex = db_api.get_workflow_execution(wf_ex.id)
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
-        self.assertEqual(2, len(wf_ex.task_executions))
+            task_execs = wf_ex.task_executions
+
+        self.assertEqual(2, len(task_execs))
         self.assertEqual(2, len(db_api.get_task_executions()))
 
         task1_ex = self._assert_single_item(
-            wf_ex.task_executions,
+            task_execs,
             name='task1',
             state=states.SUCCESS
         )
@@ -128,7 +154,7 @@ class ReverseWorkflowEngineTest(base.EngineTestCase):
         self.assertDictEqual({'result1': 'a'}, task1_ex.published)
 
         task2_ex = self._assert_single_item(
-            wf_ex.task_executions,
+            task_execs,
             name='task2',
             state=states.SUCCESS
         )
@@ -140,7 +166,7 @@ class ReverseWorkflowEngineTest(base.EngineTestCase):
 
         wf_ex = self.engine.start_workflow(
             'my_wb.wf1',
-            wf_input,
+            wf_input=wf_input,
             task_name='task4'
         )
 
@@ -175,4 +201,4 @@ class ReverseWorkflowEngineTest(base.EngineTestCase):
             wf_text
         )
 
-        self.assertIn("Task 'task1' not found", exception.message)
+        self.assertIn("Task 'task1' not found", str(exception))
