@@ -1,4 +1,5 @@
 # Copyright 2015 - Mirantis, Inc.
+# Copyright 2020 Nokia Software.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,17 +13,19 @@
 #    See the License for the specific language governing permissions and
 #    limitations under the License.
 
-import mock
+from unittest import mock
+
 from oslo_config import cfg
 
 from mistral.actions import std_actions
 from mistral.db.v2 import api as db_api
-from mistral.db.v2.sqlalchemy import models
 from mistral import exceptions as exc
-from mistral.services import actions
+from mistral.services import adhoc_actions
 from mistral.tests.unit.engine import base
 from mistral.workflow import states
-from mistral.workflow import utils as wf_utils
+
+from mistral_lib import actions as ml_actions
+from mistral_lib import exceptions as ml_exc
 
 # Use the set_default method to set value otherwise in certain test cases
 # the change in value is not permanent.
@@ -30,9 +33,8 @@ cfg.CONF.set_default('auth_enable', False, group='pecan')
 
 
 class RunActionEngineTest(base.EngineTestCase):
-    @classmethod
-    def heavy_init(cls):
-        super(RunActionEngineTest, cls).heavy_init()
+    def setUp(self):
+        super(RunActionEngineTest, self).setUp()
 
         action = """---
         version: '2.0'
@@ -72,6 +74,11 @@ class RunActionEngineTest(base.EngineTestCase):
           input:
             - some_input
 
+        nested_missing_base:
+          base: missing_base
+          input:
+            - some_input
+
         loop_action:
           base: loop_action
           base-input:
@@ -86,10 +93,8 @@ class RunActionEngineTest(base.EngineTestCase):
           input:
             - output
         """
-        actions.create_actions(action)
 
-    def tearDown(self):
-        super(RunActionEngineTest, self).tearDown()
+        adhoc_actions.create_actions(action)
 
     def test_run_action_sync(self):
         # Start action and see the result.
@@ -98,6 +103,100 @@ class RunActionEngineTest(base.EngineTestCase):
         self.assertEqual('Hello!', action_ex.output['result'])
         self.assertEqual(states.SUCCESS, action_ex.state)
 
+    def test_run_action_with_namespace(self):
+        namespace = 'test_ns'
+
+        action_text = """---
+        version: '2.0'
+
+        concat1:
+          base: std.echo
+          base-input:
+            output: <% $.left %><% $.right %>
+          input:
+            - left
+            - right
+
+        concat2:
+          base: concat1
+          base-input:
+            left: <% $.left %><% $.center %>
+            right: <% $.right %>
+          input:
+            - left
+            - center
+            - right
+        """
+
+        adhoc_actions.create_actions(action_text, namespace=namespace)
+
+        self.assertRaises(
+            exc.InvalidActionException,
+            self.engine.start_action,
+            'concat1',
+            {
+                'left': 'Hello, ',
+                'right': 'John Doe!'
+            },
+            save_result=True,
+            namespace=''
+        )
+
+        action_ex = self.engine.start_action(
+            'concat1',
+            {
+                'left': 'Hello, ',
+                'right': 'John Doe!'
+            },
+            save_result=True,
+            namespace=namespace
+        )
+
+        self.assertEqual(namespace, action_ex.workflow_namespace)
+
+        self.await_action_success(action_ex.id)
+
+        with db_api.transaction():
+            action_ex = db_api.get_action_execution(action_ex.id)
+
+            self.assertEqual(states.SUCCESS, action_ex.state)
+            self.assertEqual({'result': 'Hello, John Doe!'}, action_ex.output)
+
+        action_ex = self.engine.start_action(
+            'concat2',
+            {
+                'left': 'Hello, ',
+                'center': 'John',
+                'right': ' Doe!'
+            },
+            save_result=True,
+            namespace=namespace
+        )
+
+        self.assertEqual(namespace, action_ex.workflow_namespace)
+
+        self.await_action_success(action_ex.id)
+
+        with db_api.transaction():
+            action_ex = db_api.get_action_execution(action_ex.id)
+
+            self.assertEqual(states.SUCCESS, action_ex.state)
+            self.assertEqual('Hello, John Doe!', action_ex.output['result'])
+
+    def test_run_action_with_invalid_namespace(self):
+        # This test checks the case in which, the action with that name is
+        # not found with the given name, if an action was found with the
+        # same name in default namespace, that action will run.
+
+        action_ex = self.engine.start_action(
+            'concat',
+            {'left': 'Hello, ', 'right': 'John Doe!'},
+            save_result=True,
+            namespace='namespace'
+        )
+
+        self.assertIsNotNone(action_ex)
+
     @mock.patch.object(
         std_actions.EchoAction,
         'run',
@@ -121,10 +220,50 @@ class RunActionEngineTest(base.EngineTestCase):
 
         self.await_action_success(action_ex.id)
 
-        action_ex = db_api.get_action_execution(action_ex.id)
+        with db_api.transaction():
+            action_ex = db_api.get_action_execution(action_ex.id)
 
+            self.assertEqual(states.SUCCESS, action_ex.state)
+            self.assertEqual({'result': 'Hello!'}, action_ex.output)
+
+    def test_run_action_run_sync(self):
+        # Start action.
+        action_ex = self.engine.start_action(
+            'std.echo',
+            {'output': 'Hello!'},
+            run_sync=True
+        )
+
+        self.assertEqual('Hello!', action_ex.output['result'])
         self.assertEqual(states.SUCCESS, action_ex.state)
-        self.assertEqual({'result': 'Hello!'}, action_ex.output)
+
+    def test_run_action_save_result_and_run_sync(self):
+        # Start action.
+        action_ex = self.engine.start_action(
+            'std.echo',
+            {'output': 'Hello!'},
+            save_result=True,
+            run_sync=True
+        )
+
+        self.assertEqual('Hello!', action_ex.output['result'])
+        self.assertEqual(states.SUCCESS, action_ex.state)
+
+        with db_api.transaction():
+            action_ex = db_api.get_action_execution(action_ex.id)
+
+            self.assertEqual(states.SUCCESS, action_ex.state)
+            self.assertEqual({'result': 'Hello!'}, action_ex.output)
+
+    def test_run_action_run_sync_error(self):
+        # Start action.
+        self.assertRaises(
+            exc.InputException,
+            self.engine.start_action,
+            'std.async_noop',
+            {},
+            run_sync=True
+        )
 
     def test_run_action_async(self):
         action_ex = self.engine.start_action('std.async_noop', {})
@@ -143,23 +282,25 @@ class RunActionEngineTest(base.EngineTestCase):
 
         self.await_action_error(action_ex.id)
 
-        action_ex = db_api.get_action_execution(action_ex.id)
+        with db_api.transaction():
+            action_ex = db_api.get_action_execution(action_ex.id)
 
-        self.assertEqual(states.ERROR, action_ex.state)
-        self.assertIn('Invoke failed.', action_ex.output.get('result', ''))
+            self.assertEqual(states.ERROR, action_ex.state)
+            self.assertIn('Invoke failed.', action_ex.output.get('result', ''))
 
     @mock.patch.object(
         std_actions.AsyncNoOpAction, 'run',
-        mock.MagicMock(return_value=wf_utils.Result(error='Invoke erred.')))
+        mock.MagicMock(return_value=ml_actions.Result(error='Invoke erred.')))
     def test_run_action_async_invoke_with_error(self):
         action_ex = self.engine.start_action('std.async_noop', {})
 
         self.await_action_error(action_ex.id)
 
-        action_ex = db_api.get_action_execution(action_ex.id)
+        with db_api.transaction():
+            action_ex = db_api.get_action_execution(action_ex.id)
 
-        self.assertEqual(states.ERROR, action_ex.state)
-        self.assertIn('Invoke erred.', action_ex.output.get('result', ''))
+            self.assertEqual(states.ERROR, action_ex.state)
+            self.assertIn('Invoke erred.', action_ex.output.get('result', ''))
 
     def test_run_action_adhoc(self):
         # Start action and see the result.
@@ -196,12 +337,21 @@ class RunActionEngineTest(base.EngineTestCase):
     def test_run_action_with_missing_base(self):
         # Start action and see the result.
         self.assertRaises(
-            exc.DBEntityNotFoundError,
+            exc.InvalidActionException,
             self.engine.start_action,
             'missing_base',
             {'some_input': 'Hi'}
         )
 
+    def test_run_action_with_missing_nested_base(self):
+        # Start action and see the result.
+        self.assertRaises(
+            exc.InvalidActionException,
+            self.engine.start_action,
+            'nested_missing_base',
+            {'some_input': 'Hi'}
+        )
+
     def test_run_loop_action(self):
         # Start action and see the result.
         self.assertRaises(
@@ -223,63 +373,40 @@ class RunActionEngineTest(base.EngineTestCase):
     def test_run_action_wrong_input(self):
         # Start action and see the result.
         exception = self.assertRaises(
-            exc.InputException,
+            ml_exc.ActionException,
             self.engine.start_action,
             'std.http',
             {'url': 'Hello, ', 'metod': 'John Doe!'}
         )
 
-        self.assertIn('std.http', exception.message)
+        self.assertIn('std.http', str(exception))
 
     def test_adhoc_action_wrong_input(self):
         # Start action and see the result.
         exception = self.assertRaises(
-            exc.InputException,
+            ml_exc.ActionException,
             self.engine.start_action,
             'concat',
             {'left': 'Hello, ', 'ri': 'John Doe!'}
         )
 
-        self.assertIn('concat', exception.message)
+        self.assertIn('concat', str(exception))
 
-    # TODO(rakhmerov): This is an example of a bad test. It pins to
-    # implementation details too much and prevents from making refactoring
-    # easily. When writing tests we should make assertions about
-    # consequences, not about how internal machinery works, i.e. we need to
-    # follow "black box" testing paradigm.
-    @mock.patch('mistral.engine.actions.resolve_action_definition')
-    @mock.patch('mistral.engine.utils.validate_input')
-    @mock.patch('mistral.services.action_manager.get_action_class')
-    @mock.patch('mistral.engine.actions.PythonAction.run')
-    def test_run_action_with_kwargs_input(self, run_mock, class_mock,
-                                          validate_mock, def_mock):
-        action_def = models.ActionDefinition()
-        action_def.update({
-            'name': 'fake_action',
-            'action_class': '',
-            'attributes': {},
-            'description': '',
-            'input': '**kwargs',
-            'is_system': True,
-            'scope': 'public'
-        })
-        def_mock.return_value = action_def
-        run_mock.return_value = wf_utils.Result(data='Hello')
+    def test_run_action_with_kwargs_input(self):
+        class FakeAction(ml_actions.Action):
+            def __init__(self, **kwargs):
+                super(FakeAction, self).__init__()
 
-        class_ret = mock.MagicMock()
-        class_mock.return_value = class_ret
+                self.kwargs = kwargs
 
-        self.engine.start_action('fake_action', {'input': 'Hello'})
+            def run(self, context):
+                return ml_actions.Result(data=self.kwargs)
 
-        self.assertEqual(1, def_mock.call_count)
-        def_mock.assert_called_with('fake_action')
+        self.register_action_class('fake_action', FakeAction)
 
-        self.assertEqual(0, validate_mock.call_count)
-
-        class_ret.assert_called_once_with(input='Hello')
-
-        run_mock.assert_called_once_with(
-            {'input': 'Hello'},
-            None,
-            save=None
+        action_ex = self.engine.start_action(
+            'fake_action',
+            {'param1': 'Hello'}
         )
+
+        self.assertDictEqual({'param1': 'Hello'}, action_ex.input)
