@@ -12,17 +12,18 @@
 #    See the License for the specific language governing permissions and
 #    limitations under the License.
 
-import mock
+from unittest import mock
+
 from oslo_config import cfg
 
 from mistral.db.v2 import api as db_api
 from mistral import exceptions as exc
+from mistral.lang import parser as spec_parser
 from mistral.services import workbooks as wb_service
 from mistral.tests.unit.engine import base
-from mistral.workbook import parser as spec_parser
 from mistral.workflow import data_flow
 from mistral.workflow import states
-from mistral.workflow import utils
+from mistral_lib import actions as ml_actions
 
 
 # Use the set_default method to set value otherwise in certain test cases
@@ -195,27 +196,34 @@ class WorkflowResumeTest(base.EngineTestCase):
         wb_service.create_workbook_v2(RESUME_WORKBOOK)
 
         # Start workflow.
-        wf_ex = self.engine.start_workflow('wb.wf1', {})
+        wf_ex = self.engine.start_workflow('wb.wf1')
 
         self.await_workflow_paused(wf_ex.id)
 
-        wf_ex = db_api.get_workflow_execution(wf_ex.id)
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
+
+            task_execs = wf_ex.task_executions
 
         self.assertEqual(states.PAUSED, wf_ex.state)
-        self.assertEqual(2, len(wf_ex.task_executions))
+        self.assertEqual(2, len(task_execs))
 
         self.engine.resume_workflow(wf_ex.id)
 
-        wf_ex = db_api.get_workflow_execution(wf_ex.id)
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
-        self.assertEqual(2, len(wf_ex.task_executions))
+            self.assertEqual(2, len(wf_ex.task_executions))
 
         self.await_workflow_success(wf_ex.id)
 
-        wf_ex = db_api.get_workflow_execution(wf_ex.id)
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
+
+            task_execs = wf_ex.task_executions
 
         self.assertEqual(states.SUCCESS, wf_ex.state)
-        self.assertEqual(2, len(wf_ex.task_executions))
+        self.assertEqual(2, len(task_execs))
 
     def test_resume_reverse(self):
         wb_service.create_workbook_v2(RESUME_WORKBOOK_REVERSE)
@@ -223,18 +231,19 @@ class WorkflowResumeTest(base.EngineTestCase):
         # Start workflow.
         wf_ex = self.engine.start_workflow(
             'resume_reverse.wf',
-            {},
             task_name='task2'
         )
 
-        # Note: We need to reread execution to access related tasks.
-
         self.engine.pause_workflow(wf_ex.id)
 
-        wf_ex = db_api.get_workflow_execution(wf_ex.id)
+        with db_api.transaction():
+            # Note: We need to reread execution to access related tasks.
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
+
+            task_execs = wf_ex.task_executions
 
         self.assertEqual(states.PAUSED, wf_ex.state)
-        self.assertEqual(1, len(wf_ex.task_executions))
+        self.assertEqual(1, len(task_execs))
 
         self.engine.resume_workflow(wf_ex.id)
 
@@ -243,49 +252,60 @@ class WorkflowResumeTest(base.EngineTestCase):
         self.assertEqual(states.RUNNING, wf_ex.state)
 
         self.await_workflow_success(wf_ex.id)
-        wf_ex = db_api.get_workflow_execution(wf_ex.id)
+
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
+
+            task_execs = wf_ex.task_executions
 
         self.assertEqual(states.SUCCESS, wf_ex.state)
-        self.assertEqual(2, len(wf_ex.task_executions))
+        self.assertEqual(2, len(task_execs))
 
     def test_resume_two_branches(self):
         wb_service.create_workbook_v2(WORKBOOK_TWO_BRANCHES)
 
         # Start workflow.
-        wf_ex = self.engine.start_workflow('wb.wf1', {})
+        wf_ex = self.engine.start_workflow('wb.wf1')
 
         self.await_workflow_paused(wf_ex.id)
 
-        wf_ex = db_api.get_workflow_execution(wf_ex.id)
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
+
+            task_execs = wf_ex.task_executions
 
         self.assertEqual(states.PAUSED, wf_ex.state)
-        self.assertEqual(3, len(wf_ex.task_executions))
+        self.assertEqual(3, len(task_execs))
 
         wf_ex = self.engine.resume_workflow(wf_ex.id)
 
         self.await_workflow_success(wf_ex.id)
 
-        wf_ex = db_api.get_workflow_execution(wf_ex.id)
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
+
+            task_execs = wf_ex.task_executions
 
         self.assertEqual(states.SUCCESS, wf_ex.state)
 
         # We can see 3 tasks in execution.
-        self.assertEqual(3, len(wf_ex.task_executions))
+        self.assertEqual(3, len(task_execs))
 
     def test_resume_two_start_tasks(self):
         wb_service.create_workbook_v2(WORKBOOK_TWO_START_TASKS)
 
         # Start workflow.
-        wf_ex = self.engine.start_workflow('wb.wf1', {})
+        wf_ex = self.engine.start_workflow('wb.wf1')
 
         self.await_workflow_paused(wf_ex.id)
 
-        wf_ex = db_api.get_workflow_execution(wf_ex.id)
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
+
+            task_execs = wf_ex.task_executions
 
         self.assertEqual(states.PAUSED, wf_ex.state)
 
-        task_execs = wf_ex.task_executions
-
         # The exact number of tasks depends on which of two tasks
         # 'task1' and 'task2' completed earlier.
         self.assertGreaterEqual(len(task_execs), 2)
@@ -300,25 +320,29 @@ class WorkflowResumeTest(base.EngineTestCase):
 
         self.await_workflow_success(wf_ex.id)
 
-        wf_ex = db_api.get_workflow_execution(wf_ex.id)
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
+
+            task_execs = wf_ex.task_executions
 
         self.assertEqual(states.SUCCESS, wf_ex.state)
-        self.assertEqual(3, len(wf_ex.task_executions))
+        self.assertEqual(3, len(task_execs))
 
     def test_resume_different_task_states(self):
         wb_service.create_workbook_v2(WORKBOOK_DIFFERENT_TASK_STATES)
 
         # Start workflow.
-        wf_ex = self.engine.start_workflow('wb.wf1', {})
+        wf_ex = self.engine.start_workflow('wb.wf1')
 
         self.await_workflow_paused(wf_ex.id)
 
-        wf_ex = db_api.get_workflow_execution(wf_ex.id)
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
+
+            task_execs = wf_ex.task_executions
 
         self.assertEqual(states.PAUSED, wf_ex.state)
 
-        task_execs = wf_ex.task_executions
-
         self.assertEqual(3, len(task_execs))
 
         task2_ex = self._assert_single_item(task_execs, name='task2')
@@ -341,20 +365,23 @@ class WorkflowResumeTest(base.EngineTestCase):
             task_execution_id=task2_ex.id
         )[0]
 
-        self.engine.on_action_complete(task2_action_ex.id, utils.Result())
+        self.engine.on_action_complete(task2_action_ex.id, ml_actions.Result())
 
         self.await_workflow_success(wf_ex.id)
 
-        wf_ex = db_api.get_workflow_execution(wf_ex.id)
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
+
+            task_execs = wf_ex.task_executions
 
         self.assertEqual(states.SUCCESS, wf_ex.state, wf_ex.state_info)
-        self.assertEqual(4, len(wf_ex.task_executions))
+        self.assertEqual(4, len(task_execs))
 
     def test_resume_fails(self):
         # Start and pause workflow.
         wb_service.create_workbook_v2(WORKBOOK_DIFFERENT_TASK_STATES)
 
-        wf_ex = self.engine.start_workflow('wb.wf1', {})
+        wf_ex = self.engine.start_workflow('wb.wf1')
 
         self.await_workflow_paused(wf_ex.id)
 
@@ -386,28 +413,23 @@ class WorkflowResumeTest(base.EngineTestCase):
         }
 
         # Start workflow.
-        wf_ex = self.engine.start_workflow('wb.wf1', {}, env=env)
+        wf_ex = self.engine.start_workflow('wb.wf1', env=env)
 
         self.await_workflow_paused(wf_ex.id)
 
-        wf_ex = db_api.get_workflow_execution(wf_ex.id)
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
-        task_1_ex = self._assert_single_item(
-            wf_ex.task_executions,
-            name='task1'
-        )
+            task_execs = wf_ex.task_executions
 
-        task_2_ex = self._assert_single_item(
-            wf_ex.task_executions,
-            name='task2'
-        )
+            task_1_ex = self._assert_single_item(task_execs, name='task1')
+            task_2_ex = self._assert_single_item(task_execs, name='task2')
 
-        self.assertEqual(states.PAUSED, wf_ex.state)
-        self.assertEqual(2, len(wf_ex.task_executions))
-        self.assertDictEqual(env, wf_ex.params['env'])
-        self.assertDictEqual(env, wf_ex.context['__env'])
-        self.assertEqual(states.SUCCESS, task_1_ex.state)
-        self.assertEqual(states.IDLE, task_2_ex.state)
+            self.assertEqual(states.PAUSED, wf_ex.state)
+            self.assertEqual(2, len(task_execs))
+            self.assertDictEqual(env, wf_ex.params['env'])
+            self.assertEqual(states.SUCCESS, task_1_ex.state)
+            self.assertEqual(states.IDLE, task_2_ex.state)
 
         # Update env in workflow execution with the following.
         updated_env = {
@@ -420,19 +442,18 @@ class WorkflowResumeTest(base.EngineTestCase):
 
         self.await_workflow_success(wf_ex.id)
 
-        wf_ex = db_api.get_workflow_execution(wf_ex.id)
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
-        self.assertDictEqual(updated_env, wf_ex.params['env'])
-        self.assertDictEqual(updated_env, wf_ex.context['__env'])
-        self.assertEqual(3, len(wf_ex.task_executions))
+            task_execs = wf_ex.task_executions
 
-        # Check result of task2.
-        task_2_ex = self._assert_single_item(
-            wf_ex.task_executions,
-            name='task2'
-        )
+            self.assertDictEqual(updated_env, wf_ex.params['env'])
+            self.assertEqual(3, len(task_execs))
 
-        self.assertEqual(states.SUCCESS, task_2_ex.state)
+            # Check result of task2.
+            task_2_ex = self._assert_single_item(task_execs, name='task2')
+
+            self.assertEqual(states.SUCCESS, task_2_ex.state)
 
         # Re-read task execution, otherwise lazy loading of action executions
         # may not work.
@@ -441,15 +462,15 @@ class WorkflowResumeTest(base.EngineTestCase):
 
             task_2_result = data_flow.get_task_execution_result(task_2_ex)
 
-        self.assertEqual(updated_env['var1'], task_2_result)
+            self.assertEqual(updated_env['var1'], task_2_result)
 
-        # Check result of task3.
-        task_3_ex = self._assert_single_item(
-            wf_ex.task_executions,
-            name='task3'
-        )
+            # Check result of task3.
+            task_3_ex = self._assert_single_item(
+                task_execs,
+                name='task3'
+            )
 
-        self.assertEqual(states.SUCCESS, task_3_ex.state)
+            self.assertEqual(states.SUCCESS, task_3_ex.state)
 
         # Re-read task execution, otherwise lazy loading of action executions
         # may not work.
@@ -458,4 +479,4 @@ class WorkflowResumeTest(base.EngineTestCase):
 
             task_3_result = data_flow.get_task_execution_result(task_3_ex)
 
-        self.assertEqual(updated_env['var2'], task_3_result)
+            self.assertEqual(updated_env['var2'], task_3_result)
