@@ -13,6 +13,7 @@
 #    See the License for the specific language governing permissions and
 #    limitations under the License.
 
+from oslo_config import cfg
 from oslo_log import log as logging
 from osprofiler import profiler
 
@@ -27,6 +28,7 @@ from mistral_lib import utils
 
 
 LOG = logging.getLogger(__name__)
+CONF = cfg.CONF
 
 MAX_SEARCH_DEPTH = 5
 
@@ -46,7 +48,7 @@ class DirectWorkflowController(base.WorkflowController):
 
     __workflow_type__ = "direct"
 
-    def _get_upstream_task_executions(self, task_spec):
+    def _get_upstream_task_executions(self, task_spec, triggered_by=None):
         t_specs_names = [t_spec.get_name() for t_spec in
                          self.wf_spec.find_inbound_task_specs(task_spec)]
 
@@ -54,6 +56,14 @@ class DirectWorkflowController(base.WorkflowController):
             return []
 
         if not task_spec.get_join():
+            if triggered_by and len(triggered_by) > 0:
+                return self._get_task_executions(
+                    name=t_specs_names[0],  # not a join, has just one parent
+                    state={'in': (states.SUCCESS, states.ERROR,
+                                  states.CANCELLED)},
+                    id={'in': triggered_by},
+                    processed=True
+                )
             return self._get_task_executions(
                 name=t_specs_names[0],  # not a join, has just one parent
                 state={'in': (states.SUCCESS, states.ERROR, states.CANCELLED)},
@@ -188,10 +198,16 @@ class DirectWorkflowController(base.WorkflowController):
         ctx = {}
 
         for batch in self._find_end_task_executions_as_batches():
-            for t_ex in batch:
-                ctx = utils.merge_dicts(
-                    ctx,
-                    data_flow.evaluate_task_outbound_context(t_ex)
+            if not cfg.CONF.context_versioning.enabled:
+                for t_ex in batch:
+                    ctx = utils.merge_dicts(
+                        ctx,
+                        data_flow.evaluate_task_outbound_context(t_ex)
+                    )
+            else:
+                ctx = data_flow.evaluate_upstream_context(
+                    batch,
+                    additive_context=ctx
                 )
 
         return ctx
@@ -298,14 +314,27 @@ class DirectWorkflowController(base.WorkflowController):
 
                     result.append((name, params, 'on-error'))
 
-        if t_s == states.SUCCESS:
+        skip_is_empty = False
+        if t_s == states.SKIPPED:
+            for name, cond, params in self.wf_spec.get_on_skip_clause(t_n):
+                if not cond or expr.evaluate(cond, ctx_view):
+                    params = expr.evaluate_recursively(params, ctx_view)
+                    result.append((name, params, 'on-skip'))
+
+            # We should go to 'on-success' branch in case of
+            # skipping task with no 'on-skip' specified.
+            if len(result) == 0:
+                skip_is_empty = True
+
+        if t_s == states.SUCCESS or skip_is_empty:
             for name, cond, params in self.wf_spec.get_on_success_clause(t_n):
                 if not cond or expr.evaluate(cond, ctx_view):
                     params = expr.evaluate_recursively(params, ctx_view)
 
                     result.append((name, params, 'on-success'))
 
-        if states.is_completed(t_s) and not states.is_cancelled(t_s):
+        if states.is_completed(t_s) \
+                and not states.is_cancelled_or_skipped(t_s):
             for name, cond, params in self.wf_spec.get_on_complete_clause(t_n):
                 if not cond or expr.evaluate(cond, ctx_view):
                     params = expr.evaluate_recursively(params, ctx_view)
